#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Créé le Fri Apr  8 19:51:49 2022

@auteur: david ALBERTO
(www.astrolabe-science.fr)
"""
import matplotlib.pyplot as plt
import numpy as np
from skyfield.api import Star, load
from skyfield.data import hipparcos
from skyfield.api import N,S,E,W, wgs84 #entrer coordonnées d'un lieu
from skyfield.api import Loader #ajuster le fichier de téléchargement d'éphémérides
import datetime as dt
from pytz import timezone
import locale #dates en français

plt.rcParams["font.family"] = "Roboto Condensed"

# Paramètres réglables par l'utilisateur :
latitude=49.5# degrés
taille_points=4
pasHeures=2
date_debut=dt.datetime(1500, 6, 15, 19, 0)# année, mois, jour, heure, min
date_fin=dt.datetime(1500, 6, 16, 6)# année, mois, jour, heure, min
prop_lignes = dict(lw=1)
# -----------------------------------------------
    
locale.setlocale(locale.LC_ALL, 'fr_FR.UTF-8')#dates en français

PSD=np.pi/2#pi sur 2

# Définition des étoiles de la Petite Ourse :
with load.open(hipparcos.URL) as f:
    df = hipparcos.load_dataframe(f)

# Rechercher le numéro d'une étoile dans le catalogue Hipparcos :
    # https://vizier.u-strasbg.fr/viz-bin/VizieR?-source=1239/hip_main
# Insérer ce numéro entre crochets :
polaris = Star.from_dataframe(df.loc[11767])
beta = Star.from_dataframe(df.loc[72607])#Kochab
gamma = Star.from_dataframe(df.loc[75097])#Pherkad
zeta = Star.from_dataframe(df.loc[77055])
delta = Star.from_dataframe(df.loc[85822])
epsilon = Star.from_dataframe(df.loc[82080])
eta = Star.from_dataframe(df.loc[79822])

liste_etoiles=[polaris, delta, epsilon, zeta, eta, gamma, beta, zeta]#ordre choisi pour le tracé de l'astérisme
# -----------------------------------
load = Loader('../skyfield-ephemerides')
# planets = load('de440.bsp')# 114MB autre fichier d'éphémérides (1550-2650)
# planets = load('de422.bsp')# 623MB autre fichier d'éphémérides (-3000-3000)
planets = load('de422_excerpt.bsp')# 22MB autre fichier d'éphémérides (1450-1650)

#Paramètres du lieu d'observation :
earth = planets['earth']
lieu = earth + wgs84.latlon(latitude * N, 0.109 * W)
# -------------------------------

ts = load.timescale()

pasH=dt.timedelta(hours=pasHeures)

#Prise en compte du fuseau horaire :
zone = timezone('CET')# fuseau horaire Europe Occidentale
date_debut=zone.localize(date_debut)
date_fin=zone.localize(date_fin)
# --------------------------------------
# PARAMETRES DU GRAPHIQUE :
fig=plt.figure(figsize=(8,8),tight_layout=True, facecolor='lavender')
ax=plt.subplot(111,projection='polar', facecolor='white')
ax.set_theta_zero_location("N")
ax.set_xticks([])
ax.set_title("Rotation de la constellation \n de la Petite Ourse", fontsize=14,y=0.96, loc='left')

# --------------------------------------
#  Tracé :
x_pol, y_pol = [],[]  # coordonnées de l'étoile polaire
date = date_debut
compteur = -1
while date <= date_fin:
    if compteur==-1:# premier tracé en noir
        couleur='k'
    else:
        couleur='C'+str(compteur)
    t=ts.from_datetime(date)
    liste_X, liste_Y = [], []
    for etoile in liste_etoiles:
        astrometric = lieu.at(t).observe(etoile)
        apparent = astrometric.apparent()
        alt, az, distance = apparent.altaz()
        ha, dec, distance = apparent.hadec()
        liste_X.append(ha.radians)
        liste_Y.append(np.degrees(PSD-dec.radians)) # distance polaire de l'étoile
    ax.plot(liste_X, liste_Y, 'o-',
            c=couleur, **prop_lignes, mec='k', mew=0.4, ms=taille_points,
            label=date.strftime("%Hh%M"))
    # Étoile polaire :
    x_pol.append(liste_X[0])
    y_pol.append(liste_Y[0])
    date=date+pasH
    compteur+=1

#  positions de l'étoile polaire d'une autre couleur :
ax.scatter(x_pol, y_pol, c='gainsboro', ec='indigo', lw=0.6,
            s=taille_points*7, zorder=2, label="étoile polaire")

dist_moy = sum(y_pol)/len(y_pol)
ax.text(0.03,0.01, f"distance moyenne de la Polaire : {dist_moy:.2f}°", fontsize=8,
        transform = ax.transAxes)

ax.set_yticks(np.arange(0,20,1), minor=True)
ax.set_ylim(0, 20)
ax.set_rlabel_position(120)#position des graduations des rayons
rtick_locs = np.arange(5,20,5)
rtick_labels = ['%i°'%r for r in rtick_locs]
ax.set_rgrids(rtick_locs, rtick_labels, fontsize=10, ha='center', va='center')

ax.grid(which='major', alpha=1, linewidth=0.7)
ax.grid(which='minor', alpha=0.6, linewidth=0.3)

#  pôle nord :
ax.scatter(0,0, marker='o', c='lightsalmon', ec='k', lw=0.4, label="pôle nord céleste")
ax.text(0,0,'P', ha='center', va='center', fontsize=5)

# Mise en forme de la légende (dates):
fig.subplots_adjust(bottom=0.20)#place pour la légende
legende = ax.legend(
    loc='lower center',
    title="À partir du "+date_debut.strftime('%d %b %Y'),
    ncol=5,
    bbox_to_anchor=(0.5,0.05),
    bbox_transform=fig.transFigure,
    markerscale=1,
    )

for texte in legende.get_texts():
    texte.set_fontsize(8)

legende.legendPatch.set_linewidth(1)
legende.legendPatch.set_edgecolor('purple')



NomFichier="RotationEtoilePolaire"+date_debut.strftime("%Y")
# fig.savefig(NomFichier+"_V3.png",dpi=300)
fig.savefig(NomFichier+".pdf")
fig.savefig(NomFichier+".png",dpi=300)
